/* luafaudes_common.hpp - common functionality of all classes and functions */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2006 Bernd Opitz
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */



#ifndef LUAFAUDES_INCLUDES_H
#define LUAFAUDES_INCLUDES_H

// include lua interpreter
extern "C" {
#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"
}

// include faudes library
#include "libfaudes.h"
using namespace faudes;



// console output helper functions
#include "consolehelper.hpp"


/////////////////////////
// defines
/////////////////////////



/////////////////////////
// global variables:
/////////////////////////

extern bool comp_statenames;
extern char* comp_dotpath;

/////////////////////////////////////
// Runtime configuration functions:
/////////////////////////////////////


int Comp_Statenames_On(lua_State* L);
int Comp_Statenames_Off(lua_State* L);
int Comp_DotExecPath(lua_State* L);
void RegisterCommonFaudesFunctions(lua_State* L);

/////////////////////////////////////
// Helper functions:
/////////////////////////////////////

/**
 * Test whether index on the stack containts valid userdata of type
 * tname (string)
 * 
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 * @param tname
 *   Literal name of the userdata type
 *
 * @return
 *   true / false
 */
bool lua_isudata(lua_State* L, int index, const char* tname);


/**
 * Test whether the stack indices from min to max are valid or nil.
 * Report error otherwise /(and never return)
 * @param L
 *   lua_State*
 * @param min
 *   Minimum of index range
 * @param max
 *   Maxmum of index range
 */
void lua_checkargs(lua_State* L, int min, int max);

#endif



