/* luafaudes_observer.cpp - Lua bindings for some observer plugin functions */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2008 Thomas Moor
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */


#include "luafaudes_observer.hpp"

/////////////////////////
// some functions form observer plugin
/////////////////////////


int CalcAbstAlphObs(lua_State* L) {
	fGenerator* plant = luaL_checkgenerator(L, 1);
        fEventSet*   highin = luaL_checkeventset(L, 2);
        fEventSet*   highout = luaL_checkeventset(L, 3);
	std::map< Idx , std::set<Idx> > eventmap;
	calcAbstAlphObs(*plant,*highin,*highout,eventmap);
   
        // test case
        /*
        set<Idx> newev;
        newev.insert(3);
        newev.insert(7);
        eventmap[8]=newev;
        */

        // store relabled events as table
        lua_newtable(L); // map of sets
        // loop c map
	std::map< Idx, std::set<Idx> >::const_iterator emIt; 
        int cnt1=0;
        for(emIt=eventmap.begin(); emIt!=eventmap.end(); emIt++, cnt1++) { 
          lua_pushnumber(L, emIt->first); // key
          lua_newtable(L); // value (to become)
          // loop c set
	  std::set<Idx>::const_iterator esIt; 
          int cnt2=0;
          for(esIt=emIt->second.begin(); esIt!=emIt->second.end(); esIt++, cnt2++) { 
            lua_pushnumber(L, cnt2); // pos in set
            lua_pushnumber(L,*esIt); // index in set
            lua_settable(L,-3);
          }
          // set for value 
          lua_settable(L,-3);
	}


	return 1;
}


int InsertRelabeledEvents(lua_State* L) {
	fGenerator* gen = luaL_checkgenerator(L, 1);
        // check table ... EventSet*   highin = luaL_check table map TODO (L, 2);
        fEventSet* newhigh = NULL;
        if(luaL_iseventset(L,3)) newhigh=luaL_checkeventset(L, 3);

        // get eventmap from table
	std::map< Idx , std::set<Idx> > eventmap;

        // iterate through table of sets
	lua_pushnil(L);  
	while (lua_next(L, 2) != 0) {  // have key at -2 and value at -1
          Idx oldidx = lua_tointeger(L, -2); // retrieve old idx (key)
	  eventmap[oldidx] = set<Idx>(); // create cset at idx key
          // iterate through set of idx
  	  lua_pushnil(L);  
  	  while (lua_next(L, -2) != 0) {  // have key at -2 and value at -1
	    //printf("relab: %d -> %d\n", oldidx, lua_tointeger(L,-1));
            eventmap[oldidx].insert(lua_tointeger(L,-1)); // insert new event
            lua_pop(L, 1); // remove value ids, keep key int for next
          }
	  lua_pop(L, 1); // remove value set, keep key idx for next
	}

        // doit
	if(newhigh)
          insertRelabeledEvents(*gen,eventmap,*newhigh);
        else
          insertRelabeledEvents(*gen,eventmap);


        return 0;
}



int HelpObserver(lua_State* L) {
	std::stringstream sstr;
	sstr
    << std::endl
    << "  table CalcAbstAlphObs(Generator,EventSet,EventSet,Table)" << std::endl
    << "  nil   InsertRelabeledEvents(Generator,Table,EventSet)" << std::endl;
        DisplayText("observer-plugin Functions", sstr.str());
	return 0;
}



///////////////////////////////////////////////
// register functions from observer plugin type
///////////////////////////////////////////////


void RegisterObserverFunctions(lua_State* L) {
	// register functions:
	lua_register(L, "CalcAbstAlphObs", CalcAbstAlphObs);
	lua_register(L, "InsertRelabeledEvents", InsertRelabeledEvents);
	lua_register(L, "HelpObserver", HelpObserver);
}

